#include "trackballcamera.h"
#include <glm/ext.hpp>

#define DEFAULT_SCROLL_SPEED 0.998f
#define DEFAULT_ROTATION_SPEED 0.01f
#define DEFAULT_MOVE_SPEED 0.002f

TrackBallCamera::TrackBallCamera(float myFov, float myNear, float myFar) :
    BasicCamera(myFov, myNear, myFar)
{
    reset();
}

void TrackBallCamera::rotateCamera(float dx, float dy)
{
    glm::vec3 vert(0, 1, 0);
    m_direction = glm::rotate(m_direction, -dx*DEFAULT_ROTATION_SPEED, vert);
    m_direction = glm::rotate(m_direction, -dy*DEFAULT_ROTATION_SPEED, glm::cross(m_direction, vert));
    m_direction = glm::normalize(m_direction);
    computeView();
}

void TrackBallCamera::moveCamera(float dx, float dy)
{
    // this is absolutely not efficient, but it works
    // you can try to optimize it, but please don't break it
    glm::vec4 screenPos(m_center, 1);
    glm::mat4 MVP = m_projection * m_view;
    glm::mat4 inverseMVP = glm::inverse(MVP);
    screenPos = MVP * screenPos;
    glm::vec4 pos(dx, -dy, (m_dist/m_far)*2 - 1, 1);
    pos *= screenPos.w;
    glm::vec4 diff = (inverseMVP * glm::vec4(0, 0, pos.z, pos.w)) - (inverseMVP * pos);
    moveCamera(glm::vec3(diff));
}

void TrackBallCamera::moveCamera(const glm::vec3 &diff)
{
    m_center += diff;
    computeView();
}

void TrackBallCamera::lookAt(const glm::vec3 &pos)
{
	m_direction = pos + m_direction*m_dist - m_center;
	m_dist = glm::length(m_direction);
	m_direction = glm::normalize(m_direction);
	m_center = pos;
	computeView();
}

void TrackBallCamera::reset()
{
    m_center = glm::vec3(0, 4, 0);
    m_direction = glm::normalize(glm::vec3(0, -1, -1));
    m_dist = 20;
    computeView();
}

void TrackBallCamera::computeView()
{
    m_view = glm::lookAt(m_center - m_direction*m_dist, m_center, glm::vec3(0, 1, 0));
}

void TrackBallCamera::zoom(int nbUnits)
{
	while(nbUnits != 0)
	{
		if(nbUnits > 0)
		{
			m_dist *= DEFAULT_SCROLL_SPEED;
			--nbUnits;
		}
		else
		{
			m_dist /= DEFAULT_SCROLL_SPEED;
			++nbUnits;
		}
	}
	computeView();
}

glm::vec3 TrackBallCamera::getDefaultPxInfo()
{
    glm::vec4 screenPos(m_center, 1);
    screenPos = (m_projection * m_view) * screenPos;
    return glm::vec3(m_dist/m_far, 1/screenPos.w, 0);
}
